var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};

// src/index.ts
import { promises as fs3 } from "fs";
import { join as join2, resolve as resolve2 } from "pathe";
import serialize from "@nuxt/devalue";

// src/utils.ts
import { createHash } from "crypto";
import { promises as fs, constants as fsConstants } from "fs";
import sharp from "sharp";
function loadImage(url) {
  return sharp(decodeURIComponent(parseURL(url).pathname));
}
function parseURL(rawURL) {
  return new URL(rawURL.replace(/#/g, "%23"), "file://");
}
function generateImageID(url, args) {
  return createHash("sha256").update(url).update(JSON.stringify(args)).digest("hex").slice(0, 8) + (args.format ? `.${args.format}` : "");
}
function getAssetHash(content) {
  return createHash("sha256").update(content).digest("hex").slice(0, 8);
}
async function exists(path) {
  return await fs.access(path, fsConstants.F_OK).then(() => true, () => false);
}
function formatFor(image) {
  var _a;
  const format = (_a = image.options) == null ? void 0 : _a.formatOut;
  if (!format) {
    console.error("Could not infer image format for", image);
    throw new Error("Could not infer image format");
  }
  return format;
}
function mimeTypeFor(format) {
  if (format === "jpg")
    format = "jpeg";
  return `image/${format}`;
}

// src/api.ts
import { promises as fs2 } from "fs";
import { basename, join, resolve, relative, extname } from "pathe";
import createDebugger from "debug";
var debug = {
  load: createDebugger("image-presets:load"),
  write: createDebugger("image-presets:write"),
  total: createDebugger("image-presets:total")
};
var VIRTUAL_ID = "/@imagepresets/";
function createImageApi(config) {
  const requestedImagesById = {};
  const generatedImages = [];
  const imageHashesByFile = {};
  const imageFilenamesById = {};
  return {
    getImageById(id) {
      return requestedImagesById[id];
    },
    async waitForImages() {
      debug.total("%i image(s)", generatedImages.length);
      return await Promise.all(generatedImages);
    },
    async resolveImage(filename, params) {
      const presetName = params[config.urlParam];
      const preset = config.presets[presetName];
      debug.load("%O %s", params, filename);
      if (!preset)
        throw new Error(`vite-image-presets: Unknown image preset '${presetName}'`);
      let lastSrc;
      const imageResult = await Promise.all(preset.images.map(async (_a) => {
        var _b = _a, { srcset } = _b, source = __objRest(_b, ["srcset"]);
        return __spreadProps(__spreadValues({}, source), {
          srcset: (await Promise.all(srcset.map(async ({ condition, args, generate }) => [
            lastSrc = await getImageSrc(filename, args, generate),
            condition
          ].filter((x) => x).join(" ")))).join(", ")
        });
      }));
      const lastImage = imageResult[imageResult.length - 1];
      Object.assign(lastImage, preset.attrs);
      lastImage.src || (lastImage.src = lastSrc);
      return imageResult;
    }
  };
  async function getImageHash(filename) {
    return await (imageHashesByFile[filename] || (imageHashesByFile[filename] = loadImage(filename).toBuffer().then(getAssetHash)));
  }
  async function queueImageAndGetFilename(id, sourceFilename, image) {
    const base = basename(sourceFilename, extname(sourceFilename));
    const hash = getAssetHash(id + await getImageHash(sourceFilename));
    const format = formatFor(image);
    const filename = `${base}.${hash}.${format}`;
    generatedImages.push(writeImageFile(filename, image));
    return join(config.assetsDir, filename);
  }
  async function writeImageFile(filename, image) {
    const { cacheDir, assetsDir, outDir } = config;
    const cachedFilename = join(cacheDir, filename);
    const destFilename = join(outDir, assetsDir, filename);
    if (!await exists(cachedFilename)) {
      debug.write("%s", filename);
      await image.toFile(cachedFilename);
    }
    return {
      fileName: relative(config.outDir, destFilename),
      name: filename,
      source: await fs2.readFile(cachedFilename),
      isAsset: true,
      type: "asset"
    };
  }
  async function getImageSrc(filename, args, generate) {
    filename = resolve(config.root, filename);
    const id = generateImageID(filename, args);
    const image = requestedImagesById[id] || (requestedImagesById[id] = generate(loadImage(filename), args));
    if (config.isBuild) {
      imageFilenamesById[id] || (imageFilenamesById[id] = queueImageAndGetFilename(id, filename, image));
      return config.base + await imageFilenamesById[id];
    }
    return VIRTUAL_ID + id;
  }
}

// src/presets.ts
function widthPreset(_a) {
  var _b = _a, { widths, formats } = _b, attrs = __objRest(_b, ["widths", "formats"]);
  return {
    attrs,
    images: Object.entries(formats).map(([format, formatOptions]) => ({
      type: mimeTypeFor(format),
      srcset: widths.map((width) => ({
        condition: `${width}w`,
        args: { format, width, formatOptions },
        generate: (image, { format: format2, formatOptions: formatOptions2, width: width2 }) => image.toFormat(format2, formatOptions2).resize({ width: width2, withoutEnlargement: true })
      }))
    }))
  };
}

// src/index.ts
function ImagePresetsPlugin(presets, options) {
  let api;
  let config;
  return {
    name: "image-presets",
    enforce: "pre",
    get api() {
      return api;
    },
    async configResolved({ base, command, root, build: { outDir, assetsDir } }) {
      config = __spreadValues({
        presets,
        urlParam: "preset",
        base,
        root,
        outDir: resolve2(root, outDir),
        assetsDir,
        cacheDir: join2(root, "node_modules", ".images"),
        isBuild: command === "build"
      }, options);
      api = createImageApi(config);
      if (config.isBuild)
        await fs3.mkdir(config.cacheDir, { recursive: true });
    },
    async load(id) {
      if (!id.includes(config.urlParam))
        return;
      const { path, query } = parseId(id);
      if (!query.preset)
        return;
      const images = await api.resolveImage(path, query);
      return `export default ${serialize(images)}`;
    },
    configureServer(server) {
      server.middlewares.use((req, res, next) => {
        var _a;
        if ((_a = req.url) == null ? void 0 : _a.startsWith(VIRTUAL_ID)) {
          const [, id] = req.url.split(VIRTUAL_ID);
          const image = api.getImageById(id);
          if (!image)
            throw new Error(`vite-image-presets cannot find image with id "${id}" this is likely an internal error`);
          res.setHeader("Content-Type", `image/${formatFor(image)}`);
          res.setHeader("Cache-Control", "max-age=360000");
          return image.clone().on("error", (err) => console.error(err)).pipe(res);
        }
        next();
      });
    },
    async generateBundle(_, output) {
      const images = await api.waitForImages();
      images.forEach((asset) => {
        output[asset.fileName] = asset;
      });
    }
  };
}
function parseId(id) {
  const index = id.indexOf("?");
  if (index < 0)
    return { path: id, query: {} };
  const query = Object.fromEntries(new URLSearchParams(id.slice(index)));
  return { path: id.slice(0, index), query };
}
export {
  ImagePresetsPlugin as default,
  formatFor,
  mimeTypeFor,
  widthPreset
};
